package com.qmth.demo.api.controller;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.config.ApiProperties;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.enums.Platform;
import com.qmth.boot.core.rateLimit.annotation.RateLimit;
import com.qmth.boot.core.security.config.SecurityProperties;
import com.qmth.boot.core.uid.service.UidService;
import io.micrometer.core.annotation.Timed;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX)
@Aac(strict = false, auth = false)
public class IndexController {

    @Resource
    private ApiProperties apiProperties;

    @Resource
    private UidService uidService;

    @Resource
    private SecurityProperties securityProperties;

    @RequestMapping("/config")
    public Object config(HttpServletRequest request) {
        return request.getAttribute(ApiConstant.ATTRIBUTE_API_CONFIG);
    }

    @RequestMapping("/strict")
    @Aac(strict = true)
    public Object strict() {
        return true;
    }

    @RequestMapping("/platform")
    @Aac(platform = Platform.WEB)
    public Object platform() {
        return true;
    }

    @RequestMapping("/auth")
    @Aac(auth = true)
    public Object auth() {
        return true;
    }

    @RequestMapping("/ipAllow")
    @Aac(ipAllow = "172.16.*.*")
    public Object ipallow() {
        return true;
    }

    @RequestMapping("/property")
    public Object property() {
        return apiProperties;
    }

    @RequestMapping("/security")
    public Object security() {
        return securityProperties;
    }

    @RequestMapping("/uid")
    public Object getUid() {
        return uidService.getId();
    }

    @RequestMapping("/rateLimit")
    @Aac(rateLimit = @RateLimit(count = 1))
    public Object rateLimit(@RequestParam int sleep, @RequestParam(required = false) Boolean exception)
            throws InterruptedException {
        if (exception != null && exception) {
            throw new RuntimeException("rate limit exception!");
        }
        Thread.sleep(sleep);
        return true;
    }

    @RequestMapping("/caller")
    public Object caller(HttpServletRequest request) {
        request.setAttribute(ApiConstant.ATTRIBUTE_CALLER, String.valueOf(RandomUtils.nextInt()));
        return true;
    }

    @RequestMapping("/metrics")
    @Timed(value = "api_metrics", histogram = true)
    public Object metrics(HttpServletRequest request) {
        return true;
    }
}
