package com.qmth.demo.api.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.demo.api.db.*;
import com.qmth.demo.api.service.DemoLockService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashSet;

@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/user")
@Aac(strict = false, auth = false)
public class UserController {

    @Resource
    private UserMapper userMapper;

    @Resource
    private DemoLockService demoLockService;

    @RequestMapping("/list")
    public Object list(@Validated UserQuery query) {
        return userMapper.findByEnable(query.getEnable() != null ? query.getEnable() : true);
        //return userMapper.findByQuery(query);
    }

    @RequestMapping("/query")
    public Object query(@Validated UserQuery query) {
        query.setEnable(true);
        query.orderByNameDesc();
        return userMapper.selectPage(query,
                new QueryWrapper<User>().lambda().likeRight(query.getName() != null, User::getName, query.getName())
                        .eq(query.getEnable() != null, User::isEnable, query.getEnable())
                        .eq(query.getGender() != null, User::getGender, query.getGender())).getRecords();
    }

    @RequestMapping("/add")
    @Transactional
    public Object add(@RequestParam String name, @RequestParam Gender gender) {
        User u = new User();
        u.setName(name);
        u.setGender(gender);
        u.setEnable(true);
        u.setAddress(new Address());
        u.getAddress().setProvince("hubei");
        u.getAddress().setCity("wuhan");
        u.setHistory(new HashSet<>());
        u.getHistory().add("h1");
        u.getHistory().add("h2");
        userMapper.insert(u);
        return u;
    }

    @RequestMapping("/lock/add")
    public Object lockAdd(@RequestParam String name, @RequestParam Gender gender) {
        User u = new User();
        u.setName(name);
        u.setGender(gender);
        u.setEnable(true);
        demoLockService.insert(u);
        return userMapper.findByName(name);
    }
}
